from typing import Optional

from boaviztapi.dto.device import DeviceDTO
from boaviztapi.dto.usage import Usage
from boaviztapi.dto.usage.usage import mapper_usage
from boaviztapi.model.device import Device
from boaviztapi.model.device.userTerminal import DeviceLaptop, DeviceDesktop, DeviceTablet, DeviceSmartphone, \
    DeviceTelevision, DeviceBox, DeviceUsbStick, DeviceSmartWatch, DeviceExternalHDD, DeviceMonitor, DeviceExternalSSD
from boaviztapi.service.archetype import get_arch_component


class UserTerminal(DeviceDTO):
    usage: Optional[Usage] = None


class Laptop(UserTerminal):
    type: Optional[str] = None


class Desktop(UserTerminal):
    type: Optional[str] = None


class Tablet(UserTerminal):
    pass


class Smartphone(UserTerminal):
    pass


class Television(UserTerminal):
    type: Optional[str] = None


class Smartwatch(UserTerminal):
    pass


class Box(UserTerminal):
    pass


class UsbStick(UserTerminal):
    pass


class ExternalHDD(UserTerminal):
    pass


class Monitor(UserTerminal):
    pass


class ExternalSSD(UserTerminal):
    pass


def mapper_user_terminal(user_terminal_dto: UserTerminal, archetype) -> Device:
    if type(user_terminal_dto) == Laptop:
        model = DeviceLaptop(archetype=archetype)
        model.type.set_input(user_terminal_dto.type)
    elif type(user_terminal_dto) == Desktop:
        model = DeviceDesktop(archetype=archetype)
        model.type.set_input(user_terminal_dto.type)
    elif type(user_terminal_dto) == Tablet:
        model = DeviceTablet(archetype=archetype)
    elif type(user_terminal_dto) == Smartphone:
        model = DeviceSmartphone(archetype=archetype)
    elif type(user_terminal_dto) == Television:
        model = DeviceTelevision(archetype=archetype)
        model.type.set_input(user_terminal_dto.type)
    elif type(user_terminal_dto) == Smartwatch:
        model = DeviceSmartWatch(archetype=archetype)
    elif type(user_terminal_dto) == Box:
        model = DeviceBox(archetype=archetype)
    elif type(user_terminal_dto) == UsbStick:
        model = DeviceUsbStick(archetype=archetype)
    elif type(user_terminal_dto) == ExternalHDD:
        model = DeviceExternalHDD(archetype=archetype)
    elif type(user_terminal_dto) == ExternalSSD:
        model = DeviceExternalSSD(archetype=archetype)
    elif type(user_terminal_dto) == Monitor:
        model = DeviceMonitor(archetype=archetype)
    else:
        raise Exception("User Terminal Type not found")

    if user_terminal_dto.usage is not None:
        model.usage = mapper_usage(user_terminal_dto.usage, archetype=get_arch_component(archetype, "USAGE"))

    return model



import math
from decimal import Decimal

from boaviztapi import config

def significant_number(x):
    """
    Determine the number of significant figures for x
    """
    if x == 0:
        return 0
    int_part = int(abs(x))
    if int_part == 0:
        x = remove_unsignificant_zeros(x)
    return precision_and_scale(x)[0]

def round_based_on_min_max(val, min_val, max_val, uncertainty=config['uncertainty']):
    """
    Returns a rounded values for `val`.

    The rounding depends on the difference betwwen `min_val` and `max-val`. 
    The bigger the difference, the more agressive the rounding.

    The uncertainty is a % used to round the value.

    """
    if uncertainty == 0:
        raise ValueError("Invalid precision value, cannot be 0%")
    if max_val < min_val:
        raise ValueError("round_based_on_min_max : min must be less than max")

    # value for approx% of the min max delta
    approx = (max_val - min_val) / (100 / uncertainty)
    if approx == 0:
        return val
    significant = math.floor(math.log10(approx))

    # Mathematically, these two calculation should be equivalent, but
    # dividing by very small number and multiplying by very large number
    # cause issues with floats.
    # We avoid these issues by inverting operation based on the sign of significant
    if significant > 0:
        rounded = round(val / 10 ** significant) * 10 ** significant
    else:
        rounded = round(val / 10 ** significant) / 10 ** -significant

    return float(rounded)

def round_to_sigfig(x, significant_figures):
    """
   returns a float rounded to significant figures
   """
    return float(to_precision(x, significant_figures))


def precision_and_scale(x):
    """
    Utils function
    Determine the precision and scale of x
    """
    max_digits = 14
    int_part = int(abs(x))
    magnitude = 1 if int_part == 0 else int(math.log10(int_part)) + 1
    if magnitude >= max_digits:
        return (magnitude, 0)
    frac_part = abs(x) - int_part
    multiplier = 10 ** (max_digits - magnitude)
    frac_digits = multiplier + int(multiplier * frac_part + 0.5)
    while frac_digits % 10 == 0:
        frac_digits /= 10
    scale = int(math.log10(frac_digits))
    return (magnitude + scale, scale)


def remove_unsignificant_zeros(x):
    """
    Util function
    Remove unsignificant decimal zeros  in number like 0.'000'23
    """
    if (x > 1):
        return x
    divider = Decimal("0.000000001")
    x = Decimal(str(x))
    while x % divider == 0:
        divider = divider * 10
    exponent = -math.log10(divider) + 1
    return float(x * (Decimal(10 ** exponent)))


def to_precision(x, p):
    """
    Util function
    returns a string representation of x formatted with a precision of p

    http://randlet.com/blog/python-significant-figures-format/
    Based on the webkit javascript implementation taken from here:
    https://code.google.com/p/webkit-mirror/source/browse/JavaScriptCore/kjs/number_object.cpp
    """

    x = float(x)

    if x == 0.:
        return "0." + "0" * (p - 1)

    out = []

    if x < 0:
        out.append("-")
        x = -x

    e = int(math.log10(x))
    tens = math.pow(10, e - p + 1)
    n = math.floor(x / tens)

    if n < math.pow(10, p - 1):
        e = e - 1
        tens = math.pow(10, e - p + 1)
        n = math.floor(x / tens)

    if abs((n + 1.) * tens - x) <= abs(n * tens - x):
        n = n + 1

    if n >= math.pow(10, p):
        n = n / 10.
        e = e + 1

    m = "%.*g" % (p, n)

    if e < -2 or e >= p:
        out.append(m[0])
        if p > 1:
            out.append(".")
            out.extend(m[1:p])
        out.append('e')
        if e > 0:
            out.append("+")
        out.append(str(e))
    elif e == (p - 1):
        out.append(m)
    elif e >= 0:
        out.append(m[:e + 1])
        if e + 1 < len(m):
            out.append(".")
            out.extend(m[e + 1:])
    else:
        out.append("0.")
        out.extend(["0"] * -(e + 1))
        out.append(m)

    return "".join(out)


from boaviztapi.model.boattribute import Boattribute
from boaviztapi.model.impact import Assessable
from boaviztapi.model.usage import ModelUsage
from boaviztapi.service.archetype import get_arch_value, get_arch_component


class Component(Assessable):
    NAME = "COMPONENT"

    def __init__(self, archetype=None, **kwargs):
        super().__init__(**kwargs)
        self.impact_factor = {}
        self.archetype = archetype
        self.units = Boattribute(
            default=get_arch_value(archetype, 'units', 'default', default=1),
            min=get_arch_value(archetype, 'units', 'min'),
            max=get_arch_value(archetype, 'units', 'max')
        )
        self._usage = None

    def __iter__(self):
        for attr, value in self.__dict__.items():
            yield attr, value

    @property
    def usage(self) -> ModelUsage:
        if self._usage is None:
            self._usage = ModelUsage(archetype=get_arch_component(self.archetype, "USAGE"))
        return self._usage

    @usage.setter
    def usage(self, value: int) -> None:
        self._usage = value



import requests

CLOUD_URL="http://localhost:5000/v1/cloud/instance"
PROVIDERS_URL=f"{CLOUD_URL}/all_providers"
VALID_INSTANCE_TYPE="dev1-l"

def main():
    resp = requests.get(PROVIDERS_URL)

    if not resp.ok:
        print(f"Providers request failed: {resp.json()}")
        exit(1)

    if "scaleway" in resp.json():
        print("Found Scaleway in list of providers")
    else:
        print("Did not find Scaleway in list of providers: {resp.status_code}")
        exit(1)

    url_params = {
        "provider": "scaleway",
        "instance_type": VALID_INSTANCE_TYPE,
        "verbose": True,
        "duration": "100",
        }

    resp = requests.get(CLOUD_URL, params=url_params)
    if resp.ok:
        print(f"Got response to impact request for type {VALID_INSTANCE_TYPE}")
    else:
        print("Did not find Scaleway in list of providers: {resp.status_code}")
        exit(1)

    resp_body = resp.json()
    for impact_name, impact in resp_body["impacts"].items():
        print(f"\nGot {impact_name}: {impact}")

    cpu = resp_body["verbose"]["CPU-1"]["model_range"]["value"]
    print(f"\nGot CPU model: {cpu}")

if __name__ == "__main__":
    main()



import os
from typing import Optional

import pandas as pd

from boaviztapi import config
from boaviztapi.dto.component import ComponentDTO
from boaviztapi.dto.usage import Usage
from boaviztapi.dto.usage.usage import mapper_usage
from boaviztapi.model.component import ComponentRAM
from boaviztapi.service.archetype import get_component_archetype

_ram_df = pd.read_csv(os.path.join(os.path.dirname(__file__), '../../data/crowdsourcing/ram_manufacture.csv'))


class RAM(ComponentDTO):
    capacity: Optional[int] = None
    density: Optional[float] = None
    process: Optional[float] = None
    manufacturer: Optional[str] = None
    model: Optional[str] = None


def mapper_ram(ram_dto: RAM, archetype=get_component_archetype(config["default_ram"], "ram")) -> ComponentRAM:
    ram_component = ComponentRAM(archetype=archetype)
    ram_component.usage = mapper_usage(ram_dto.usage or Usage(), archetype=archetype.get("USAGE"))

    if ram_dto.units is not None:
        ram_component.units.set_input(ram_dto.units)

    if ram_dto.density is not None:
        ram_component.density.set_input(ram_dto.density)

    if ram_dto.capacity is not None:
        ram_component.capacity.set_input(ram_dto.capacity)

    if ram_dto.manufacturer is not None:
        ram_component.manufacturer.set_input(ram_dto.manufacturer)

    if ram_dto.process is not None:
        ram_component.process.set_input(ram_dto.process)

    return ram_component




from dataclasses import dataclass
from typing import Optional

import boaviztapi.utils.roundit as rd
from boaviztapi import config

WARNING_IMPORTANT_UNCERTAINTY = ("Uncertainty from technical characteristics is very important. Results should be interpreted with caution (see min and max values)")
NOT_IMPLEMENTED = 'not implemented'


@dataclass
class ImpactCriteria:
    name: str
    unit: str
    description: str
    method: Optional[str] = None


class Impact:
    def __init__(self, **kwargs):
        self.value = 0
        self.min = 0
        self.max = 0
        self.warnings = []

        for attr, val in kwargs.items():
            if val is not None:
                self.__setattr__(attr, val)

    def add_warning(self, warn):
        if warn not in self.warnings:
            self.warnings.append(warn)

    def to_json(self):
        json = {"value": self.rounded_value()}
        if self.min or self.min == 0: json['min'] = self.rounded_min()
        if self.max or self.max == 0: json['max'] = self.rounded_max()
        if self.warnings: json['warnings'] = sorted(self.warnings)

        return json

    def rounded_value(self):
        rd_value = rd.round_based_on_min_max(self.value, self.min, self.max)
        nb_sig_fig = rd.significant_number(rd_value)

        if nb_sig_fig > config["max_sig_fig"]:
            return rd.round_to_sigfig(rd_value, config["max_sig_fig"])
        elif rd_value == 0:
            self.add_warning(WARNING_IMPORTANT_UNCERTAINTY)
            return rd.round_to_sigfig(self.value, config["min_sig_fig"])
        else:
            return rd_value

    def rounded_min(self):
        return rd.round_to_sigfig(self.min, config["max_sig_fig"])

    def rounded_max(self):
        return rd.round_to_sigfig(self.max, config["max_sig_fig"])

    def allocate(self, duration, life_time):
        if duration > life_time.value:
            allocation_ratio, allocation_ratio_min, allocation_ratio_max = 1, 1, 1
        else:
            allocation_ratio = duration / life_time.value
            allocation_ratio_min = duration / life_time.max
            allocation_ratio_max = duration / life_time.min

        self.value = self.value * allocation_ratio
        self.min = self.min * allocation_ratio_min
        self.max = self.max * allocation_ratio_max



GWP = ImpactCriteria(name="gwp", unit="kgCO2eq", description="Total climate change")
ADP = ImpactCriteria(name="adp", unit="kgSbeq", description="Use of minerals and fossil ressources")
PE = ImpactCriteria(name="pe", unit="MJ", description="Consumption of primary energy")
GWPPb = ImpactCriteria(name="gwppb", unit="kg CO2 eq.", method="PEF",
                       description="Climate change - Contribution of biogenic emissions")
GWPPf = ImpactCriteria(name="gwppf", unit="kg CO2 eq.", method="PEF",
                       description="Climate change - Contribution of fossil fuel emissions")
GWPPlu = ImpactCriteria(name="gwpplu", unit="kg CO2 eq.", method="PEF",
                        description="Climate change - Contribution of emissions from land use change")
IR = ImpactCriteria(name="ir", unit="kg U235 eq.", method="PEF", description="Emissions of radionizing substances")
LU = ImpactCriteria(name="lu", unit="No dimension", method="PEF", description="Land use")
ODP = ImpactCriteria(name="odp", unit="kg CFC-11 eq.", method="PEF", description="Depletion of the ozone layer")
PM = ImpactCriteria(name="pm", unit="Disease occurrence", method="PEF", description="Fine particle emissions")
POCP = ImpactCriteria(name="pocp", unit="kg NMVOC eq.", method="PEF", description="Photochemical ozone formation")
WU = ImpactCriteria(name="wu", unit="m3 eq.", method="PEF", description="Use of water resources")
MIPS = ImpactCriteria(name="mips", unit="kg", description="Material input per unit of services")
ADPe = ImpactCriteria(name="adpe", unit="kg SB eq.", method="PEF", description="Use of mineral and metal resources")
ADPf = ImpactCriteria(name="adpf", unit="MJ", method="PEF", description="Use of fossil resources (including nuclear)")
AP = ImpactCriteria(name="ap", unit="mol H+ eq.", method="PEF", description="Acidification")
CTUe = ImpactCriteria(name="ctue", unit="CTUe", method="PEF", description="Freshwater ecotoxicity")
CTUh_c = ImpactCriteria(name="ctuh_c", unit="CTUh", method="PEF", description="Human Toxicity - Carcinogenic Effects")
CTUh_nc = ImpactCriteria(name="ctuh_nc", unit="CTUh", method="PEF",
                         description="Human toxicity - non-carcinogenic effects")
Epf = ImpactCriteria(name="epf", unit="kg P eq.", method="PEF", description="Eutrophication of freshwater")
Epm = ImpactCriteria(name="epm", unit="kg N eq.", method="PEF", description="Eutrophication of marine waters")
Ept = ImpactCriteria(name="ept", unit="mol N eq.", method="PEF", description="Terrestrial eutrophication")
FW = ImpactCriteria(name="fw", unit="m3", method="", description="Net use of freshwater")

IMPACT_CRITERIAS = {"gwp": GWP, "adp": ADP, "pe": PE, "gwppb": GWPPb, "gwppf": GWPPf, "gwpplu": GWPPlu, "ir": IR,
                    "lu": LU, "odp": ODP, "pm": PM, "pocp": POCP, "wu": WU, "mips": MIPS, "adpe": ADPe, "adpf": ADPf,
                    "ap": AP, "ctue": CTUe, "CTUh_c": CTUh_c, "CTUh_nc": CTUh_nc, "epf": Epf, "epm": Epm, "ept": Ept,
                    "fw": FW}

EMBEDDED = "embedded"
USE = "use"

IMPACT_PHASES = [EMBEDDED, USE]


class ImpactFactor:
    def __init__(self, **kwargs):
        self.value = 0
        self.min = 0
        self.max = 0
        for attr, val in kwargs.items():
            if val is not None:
                self.__setattr__(attr, val)


class Assessable:
    def __init__(self, **kwargs):
        self._impacts = {}

    def get_impacts(self, selected_criteria):
        result = {}
        for criteria in selected_criteria:
            result[criteria] = {}
            result[criteria]["unit"] = IMPACT_CRITERIAS[criteria].unit
            result[criteria]["description"] = IMPACT_CRITERIAS[criteria].description
            for phase in IMPACT_PHASES:
                if criteria not in self._impacts or phase not in self._impacts[criteria] or self._impacts[criteria][phase] is None:
                    result[criteria][phase] = NOT_IMPLEMENTED
                else:
                    result[criteria][phase] = self._impacts[criteria][phase].to_json()
        return result
    @property
    def impacts(self):
        return self._impacts

    @impacts.setter
    def impacts(self, impacts):
        self._impacts = impacts

    def add_impacts(self, impact, criteria, phase):
        if criteria not in self._impacts:
            self._impacts[criteria] = {}
            self._impacts[criteria]["unit"] = IMPACT_CRITERIAS[criteria].unit
            self._impacts[criteria]["description"] = IMPACT_CRITERIAS[criteria].description
        if phase not in self._impacts[criteria]:
            self._impacts[criteria][phase] = {}
        self._impacts[criteria][phase] = impact





import os

import numpy as np
import pandas as pd

from boaviztapi import data_dir
from boaviztapi.model.impact import IMPACT_CRITERIAS

_electricity_emission_factors_df = pd.read_csv(os.path.join(data_dir, 'crowdsourcing/electrical_mix.csv'))
_iot_impact_factors_df = pd.read_csv(os.path.join(data_dir, 'crowdsourcing/iot_factors_tide.csv'), header=1)


def electical_mix_jsonify(electrical_mix):
    json = {"min-max": {}}
    for country_code in electrical_mix.columns:
        if country_code == "english_name" or country_code == "french_name" or country_code == "unit" or country_code == "source" or country_code == "priority" or country_code == "reference_year" or country_code == "name":
            continue
        if country_code not in json:
            json[country_code] = {}
            json[country_code]['country'] = electrical_mix[country_code][0]
        for index, row in electrical_mix.iterrows():
            if row["name"] == "renewable_energy" or row["name"] == "country":
                continue
            if np.isnan(float(row[country_code])):
                continue
            if row["name"] in json[country_code]:
                if row["priority"] > json[country_code][row["name"]]["priority"]:
                    continue

            json[country_code][row["name"]] = {}
            json[country_code][row["name"]]["unit"] = row["unit"]
            json[country_code][row["name"]]["source"] = row["source"]
            json[country_code][row["name"]]["priority"] = row["priority"]
            json[country_code][row["name"]]["reference_year"] = row["reference_year"]
            json[country_code][row["name"]]["value"] = float(row[country_code])

            if row["name"] not in json["min-max"]:
                json["min-max"][row["name"]] = {}
                json["min-max"][row["name"]]["min"] = float(row[country_code])
                json["min-max"][row["name"]]["max"] = float(row[country_code])
            else:
                if float(row[country_code]) < json["min-max"][row["name"]]["min"]:
                    json["min-max"][row["name"]]["min"] = float(row[country_code])
                if float(row[country_code]) > json["min-max"][row["name"]]["max"]:
                    json["min-max"][row["name"]]["max"] = float(row[country_code])

    return {"electricity": json}


def iot_jsonify(df):
    json = {"IoT": {}}
    df[['functional_block', 'hsl_level']] = df['api_name'].str.split(' - ', expand=True)
    df.index.names = ['api_name']
    for functional_block in df['functional_block'].unique():
        if functional_block == "Name" or functional_block == "api_name" or functional_block == "Unit":
            continue
        json["IoT"][functional_block] = {}
        for hsl_level in df[df["functional_block"] == functional_block]['hsl_level'].unique():
            if hsl_level is None:
                continue
            json["IoT"][functional_block][hsl_level] = {}
            json["IoT"][functional_block][hsl_level]["manufacture"] = {}
            json["IoT"][functional_block][hsl_level]["eol"] = {}
            json["IoT"][functional_block][hsl_level]["source"] = "Adapted by Tide from 'Assessing the embodied carbon footprint of IoT edge devices with a bottom-up life-cycle approach', 2021; Thibault Pirson et David Bol (Université catholique de Louvain, ICTEAM/ECS, Louvain-la-Neuve, Belgique)"

            for impact in IMPACT_CRITERIAS:
                if df[(df["functional_block"] == functional_block) & (df["hsl_level"] == hsl_level)].get(f"eol-{impact.name}") is None:
                    continue
                json["IoT"][functional_block][hsl_level]["manufacture"][impact.name] = float(df[(df["functional_block"] == functional_block) & (df["hsl_level"] == hsl_level)].get(f"fab-{impact.name}").iloc[0])
                json["IoT"][functional_block][hsl_level]["eol"][impact.name] = float(df[(df["functional_block"] == functional_block) & (df["hsl_level"] == hsl_level)].get(f"eol-{impact.name}").iloc[0])

    return json

if __name__ == '__main__':
    print(iot_jsonify(_iot_impact_factors_df))



#!/usr/bin/env python3

import requests
from shutil import copyfile
from csv import reader, writer
from pprint import pprint
import json
import pandas as pd
import os

def clear():
    """
    Clears the terminal screen and scroll back to present
    the user with a nice clean, new screen. Useful for managing
    menu screens in terminal applications.
    """
    os.system('cls' if os.name == 'nt' else 'echo -e \\\\033c')
    
AWS_INSTANCES = "../../archetypes/cloud/aws.csv"
AZURE_INSTANCES = "../../archetypes/cloud/azure.csv"
BOAVIZTAPI_BASE_URL = "https://api.boavizta.org/v1/cloud/"
BOAVIZTAPI_LOCAL_CONTAINER = "http://0.0.0.0:5000/v1"
RESULT_FILE = "result.csv"
if "GEN_IMPACT_PROVIDER" in os.environ and os.environ["GEN_IMPACT_PROVIDER"] == "aws":
    RESULT_FILE = "aws_result.csv"

if "GEN_IMPACT_PROVIDER" in os.environ and os.environ["GEN_IMPACT_PROVIDER"] == "aws":
    base_data = pd.read_csv(AWS_INSTANCES)
else:
    base_data = pd.read_csv(AZURE_INSTANCES)

def get_instance_impact(instance_name, usage_hours, usage_location, load_percentage):
    """Returns impacts json object for `instance_name`, by querying the BoaviztAPI located at BOAVIZTAPI_LOCAL_CONTAINER.

    Keyword arguments:
    instance_name -- str, the name/id of the instance
    usage_hours -- int, how many hours of runtime should be considered to compute impacts
    usage_location -- str, three-letters code identifying the country and electricity mix for runtime, see columns in https://github.com/Boavizta/boaviztapi/blob/main/boaviztapi/data/crowdsourcing/electrical_mix.csv
    load_percentage -- int, what average percentage of CPU time usage should be considered for impacts calculation
    """
    provider = "azure"
    if "GEN_IMPACT_PROVIDER" in os.environ and os.environ["GEN_IMPACT_PROVIDER"] == "aws":
        provider = "aws"
    if usage_hours > 0:
        query = {"provider": provider, "verbose": "true", "duration": usage_hours}
    else:
        query = {"provider": provider, "verbose": "true"}
    data = {
      "provider": provider,
      "instance_type": f"{instance_name}",
      "verbose": "true",
      "usage": {
        "usage_location": f"{usage_location}",
        "time_workload": [
          {
            "time_percentage": 100,
            "load_percentage": f"{load_percentage}"
          }
        ]
      }
    }
    impact_request = requests.post(
        f"{BOAVIZTAPI_LOCAL_CONTAINER}/cloud/instance",
        data=json.dumps(data),
        params=query
    )
    return impact_request.json()

def make_instance_impacts():
    """
    Reads AZURE_INSTANCES file, adds columns for impact, writes RESULT_FILE.
    """
    usage_locations = ["FRA", "GBR", "USA"] 
    load_averages = [0, 50, 100]

    final_data = pd.concat(
        [base_data, pd.DataFrame({
            "gwp_manufacturing_1h_impact": [],
            "gwp_manufacturing_total_host_lifetime_impact": []
        })], axis=1
    )
    for load in load_averages:
        final_data = pd.concat(
            [final_data,
            pd.DataFrame({
                "energy_1h_load{}".format(load): []
            })], axis=1
        )
        for location in usage_locations:
            final_data = pd.concat(
                [final_data, 
                pd.DataFrame({
                    "gwp_use_1h_load{}_{}".format(load, location): [],
                })], axis=1
            )
    
    for instance in base_data[["id"]].iterrows():
        clear()
        completion_percentage = round(instance[0] / len(base_data.index) * 100.0, 1)
        print("Building data... {}%".format(completion_percentage))
        instance_name = instance[1]["id"].split(",")[0]
        if completion_percentage > 50:
            final_data.to_csv(RESULT_FILE)
        impacts = {}
        impacts_total_lifetime = get_instance_impact(instance_name, -1, "GBR", 0)
        for location in usage_locations:
            impacts[location] = {}
            for load in load_averages:
                impacts[location][load] = get_instance_impact(instance_name, 1, location, load)
                final_data["gwp_use_1h_load{}_{}".format(load, location)].values[instance[0]] = impacts[location][load]["impacts"]["gwp"]["use"]["value"]
        for load in load_averages:
            final_data["energy_1h_load{}".format(load)].values[instance[0]] = impacts[usage_locations[0]][load]["verbose"]["avg_power"]["value"]
        final_data["gwp_manufacturing_1h_impact"].values[instance[0]] = impacts[usage_locations[0]][load_averages[0]]["impacts"]["gwp"]["embedded"]["value"]
        final_data["gwp_manufacturing_total_host_lifetime_impact"].values[instance[0]] = impacts_total_lifetime["impacts"]["gwp"]["embedded"]["value"]


    final_data.to_csv(RESULT_FILE, index=False)
    print("Results written in {}".format(RESULT_FILE))

if __name__ == "__main__":
    make_instance_impacts()



from typing import Optional, List

from boaviztapi.dto.component.other import FunctionalBlock
from boaviztapi.dto.device import DeviceDTO
from boaviztapi.dto.usage import Usage
from boaviztapi.dto.usage.usage import mapper_usage
from boaviztapi.model.component.functional_block import get_functional_block
from boaviztapi.model.device.iot import DeviceIoT, ComponentFunctionalBlock
from boaviztapi.service.archetype import get_arch_component


class IoT(DeviceDTO):
    functional_blocks: Optional[List[FunctionalBlock]] = []
    usage: Optional[Usage] = None


def mapper_iot_device(dto_iot: IoT, archetype):
    model = DeviceIoT(archetype=archetype)

    for functional_block in dto_iot.functional_blocks:
        class_functional_block = get_functional_block(functional_block.type.upper())
        model_functional_block = class_functional_block(archetype=get_arch_component(archetype, class_functional_block.NAME))
        model_functional_block.hsl_level.set_input(functional_block.hsl_level)
        model.add_functional_block(model_functional_block)

    if dto_iot.usage is not None:
        model.usage = mapper_usage(dto_iot.usage, archetype=get_arch_component(archetype, "USAGE"))

    return model



import os
from typing import List, Union, Optional

from fastapi import APIRouter, Query, Body, HTTPException

from boaviztapi import config, data_dir
from boaviztapi.dto.device.user_terminal import UserTerminal, mapper_user_terminal, Laptop, Desktop, Smartphone, \
    Television, Tablet, Box
from boaviztapi.routers.openapi_doc.descriptions import all_archetype_user_terminals, all_terminal_categories, \
    get_archetype_config_desc, terminal_description
from boaviztapi.routers.openapi_doc.examples import end_user_terminal
from boaviztapi.service.archetype import get_user_terminal_archetype, get_device_archetype_lst_with_type
from boaviztapi.service.impacts_computation import compute_impacts
from boaviztapi.service.verbose import verbose_device

terminal_router = APIRouter(
    prefix='/v1/terminal',
    tags=['terminal']
)


@terminal_router.get('/all',
                     description=all_terminal_categories)
async def terminal_get_all_categories():
    return {
        "laptop": "v1/terminal/laptop",
        "desktop": "v1/terminal/desktop",
        "smartphone": "v1/terminal/smartphone",
        "television": "v1/terminal/television",
        "tablet": "v1/terminal/tablet",
        "box": "v1/terminal/box"
    }


@terminal_router.get('/laptop/archetypes',
                     description=all_archetype_user_terminals)
async def laptop_get_all_archetype_name():
    return get_all_archetype_name('laptop')


@terminal_router.get('/laptop/archetype_config',
                     description=get_archetype_config_desc)
async def laptop_get_archetype_config(archetype: str = Query(example=config["default_laptop"])):
    return get_archetype_config(archetype)


@terminal_router.post('/laptop', description=terminal_description)
async def laptop_impact(laptop: Laptop = Body(None, example=end_user_terminal),
                        verbose: bool = True,
                        duration: Optional[float] = config["default_duration"],
                        archetype: str = config["default_laptop"],
                        criteria: List[str] = Query(config["default_criteria"])):
    return await user_terminal_impact(user_terminal_dto=laptop,
                                      verbose=verbose,
                                      duration=duration,
                                      criteria=criteria,
                                      archetype=archetype)


@terminal_router.get('/laptop', description=terminal_description)
async def laptop_impact(archetype: str = config["default_laptop"],
                        verbose: bool = True,
                        duration: Optional[float] = config["default_duration"],
                        criteria: List[str] = Query(config["default_criteria"])):
    return await user_terminal_impact(user_terminal_dto=Laptop(),
                                      verbose=verbose,
                                      duration=duration,
                                      criteria=criteria,
                                      archetype=archetype)


@terminal_router.get('/desktop/archetypes',
                     description=all_archetype_user_terminals)
async def desktop_get_all_archetype_name():
    return get_all_archetype_name('desktop')


@terminal_router.get('/desktop/archetype_config',
                     description=get_archetype_config_desc)
async def desktop_get_archetype_config(archetype: str = Query(example=config["default_desktop"])):
    return get_archetype_config(archetype)


@terminal_router.post('/desktop', description=terminal_description)
async def desktop_impact(desktop: Desktop = Body(None, example=end_user_terminal),
                         verbose: bool = True,
                         duration: Optional[float] = config["default_duration"],
                         archetype: str = config["default_desktop"],
                         criteria: List[str] = Query(config["default_criteria"])):
    return await user_terminal_impact(user_terminal_dto=desktop,
                                      verbose=verbose,
                                      duration=duration,
                                      criteria=criteria,
                                      archetype=archetype)


@terminal_router.get('/desktop', description=terminal_description)
async def desktop_impact(archetype: str = config["default_desktop"],
                         verbose: bool = True,
                         duration: Optional[float] = config["default_duration"],
                         criteria: List[str] = Query(config["default_criteria"])):
    return await user_terminal_impact(user_terminal_dto=Desktop(),
                                      verbose=verbose,
                                      duration=duration,
                                      criteria=criteria,
                                      archetype=archetype)


@terminal_router.get('/smartphone/archetypes',
                     description=all_archetype_user_terminals)
async def smartphone_get_all_archetype_name():
    return get_all_archetype_name('smartphone')


@terminal_router.get('/smartphone/archetype_config',
                     description=get_archetype_config_desc)
async def smartphone_get_archetype_config(archetype: str = Query(example=config["default_smartphone"])):
    return get_archetype_config(archetype)


@terminal_router.post('/smartphone', description=terminal_description)
async def smartphone_impact(smartphone: Smartphone = Body(None, example=end_user_terminal),
                            verbose: bool = True,
                            duration: Optional[float] = config["default_duration"],
                            archetype: str = config["default_smartphone"],
                            criteria: List[str] = Query(config["default_criteria"])):
    return await user_terminal_impact(user_terminal_dto=smartphone,
                                      verbose=verbose,
                                      duration=duration,
                                      criteria=criteria,
                                      archetype=archetype)


@terminal_router.get('/smartphone', description=terminal_description)
async def smartphone_impact(archetype: str = config["default_smartphone"],
                            verbose: bool = True,
                            duration: Optional[float] = config["default_duration"],
                            criteria: List[str] = Query(config["default_criteria"])):
    return await user_terminal_impact(user_terminal_dto=Smartphone(),
                                      verbose=verbose,
                                      duration=duration,
                                      criteria=criteria,
                                      archetype=archetype)


@terminal_router.get('/tablet/archetypes',
                     description=all_archetype_user_terminals)
async def tablet_get_all_archetype_name():
    return get_all_archetype_name('tablet')


@terminal_router.get('/tablet/archetype_config',
                     description=get_archetype_config_desc)
async def tablet_get_archetype_config(archetype: str = Query(example=config["default_tablet"])):
    return get_archetype_config(archetype)


@terminal_router.post('/tablet', description=terminal_description)
async def tablet_impact(tablet: Tablet = Body(None, example=end_user_terminal),
                        verbose: bool = True,
                        duration: Optional[float] = config["default_duration"],
                        archetype: str = config["default_tablet"],
                        criteria: List[str] = Query(config["default_criteria"])):
    return await user_terminal_impact(user_terminal_dto=tablet,
                                      verbose=verbose,
                                      duration=duration,
                                      criteria=criteria,
                                      archetype=archetype)


@terminal_router.get('/tablet', description=terminal_description)
async def tablet_impact(archetype: str = config["default_tablet"],
                        verbose: bool = True,
                        duration: Optional[float] = config["default_duration"],
                        criteria: List[str] = Query(config["default_criteria"])):
    return await user_terminal_impact(user_terminal_dto=Tablet(),
                                      verbose=verbose,
                                      duration=duration,
                                      criteria=criteria,
                                      archetype=archetype)


@terminal_router.get('/television/archetypes',
                     description=all_archetype_user_terminals)
async def television_get_all_archetype_name():
    return get_all_archetype_name('television')


@terminal_router.get('/television/archetype_config',
                     description=get_archetype_config_desc)
async def television_get_archetype_config(archetype: str = Query(example=config["default_television"])):
    return get_archetype_config(archetype)


@terminal_router.post('/television', description=terminal_description)
async def television_impact(television: Television = Body(None, example=end_user_terminal),
                            verbose: bool = True,
                            duration: Optional[float] = config["default_duration"],
                            archetype: str = config["default_television"],
                            criteria: List[str] = Query(config["default_criteria"])):
    return await user_terminal_impact(user_terminal_dto=television,
                                      verbose=verbose,
                                      duration=duration,
                                      criteria=criteria,
                                      archetype=archetype)


@terminal_router.get('/television', description=terminal_description)
async def television_impact(archetype: str = config["default_television"],
                            verbose: bool = True,
                            duration: Optional[float] = config["default_duration"],
                            criteria: List[str] = Query(config["default_criteria"])):
    return await user_terminal_impact(user_terminal_dto=Television(),
                                      verbose=verbose,
                                      duration=duration,
                                      criteria=criteria,
                                      archetype=archetype)


@terminal_router.get('/box/archetypes',
                     description=all_archetype_user_terminals)
async def box_get_all_archetype_name():
    return get_all_archetype_name('box')


@terminal_router.get('/box/archetype_config',
                     description=get_archetype_config_desc)
async def box_get_archetype_config(archetype: str = Query(example=config["default_box"])):
    return get_archetype_config(archetype)


@terminal_router.post('/box', description=terminal_description)
async def box_impact(box: Box = Body(None, example=end_user_terminal),
                     verbose: bool = True,
                     duration: Optional[float] = config["default_duration"],
                     archetype: str = config["default_box"],
                     criteria: List[str] = Query(config["default_criteria"])):
    return await user_terminal_impact(user_terminal_dto=box,
                                      verbose=verbose,
                                      duration=duration,
                                      criteria=criteria,
                                      archetype=archetype)


@terminal_router.get('/box', description=terminal_description)
async def box_impact(archetype: str = config["default_box"],
                     verbose: bool = True,
                     duration: Optional[float] = config["default_duration"],
                     criteria: List[str] = Query(config["default_criteria"])):
    return await user_terminal_impact(user_terminal_dto=Box(),
                                      verbose=verbose,
                                      duration=duration,
                                      criteria=criteria,
                                      archetype=archetype)


async def user_terminal_impact(user_terminal_dto: UserTerminal,
                               archetype: str,
                               verbose: bool,
                               duration: Optional[float] = config["default_duration"],
                               criteria: List[str] = Query(config["default_criteria"])) -> dict:
    archetype_config = get_user_terminal_archetype(archetype)

    if not archetype_config:
        raise HTTPException(status_code=404, detail=f"{archetype} not found")

    device = mapper_user_terminal(user_terminal_dto, archetype=archetype_config)

    if duration is None:
        duration = device.usage.hours_life_time.value

    impacts = compute_impacts(model=device, selected_criteria=criteria, duration=duration)

    if verbose:
        return {
            "impacts": impacts,
            "verbose": verbose_device(device, selected_criteria=criteria, duration=duration)
        }

    return {"impacts": impacts}


def get_all_archetype_name(name: str):
    result = get_device_archetype_lst_with_type(os.path.join(data_dir, 'archetypes/user_terminal.csv'), name.lower())
    if not result:
        return None
    return result


def get_archetype_config(archetype: str):
    result = get_user_terminal_archetype(archetype)
    if not result:
        raise HTTPException(status_code=404, detail=f"{archetype} not found")
    return result



from .component import Component
from .cpu import ComponentCPU
from .ram import ComponentRAM
from .hdd import ComponentHDD
from .ssd import ComponentSSD
from .motherboard import ComponentMotherboard
from .power_supply import ComponentPowerSupply
from .case import ComponentCase
from .assembly import ComponentAssembly



import os

import pandas as pd

from boaviztapi import config, data_dir
from boaviztapi.model.boattribute import Boattribute
from boaviztapi.model.component.component import Component
from boaviztapi.service.archetype import get_component_archetype, get_arch_value
from boaviztapi.utils.fuzzymatch import fuzzymatch_attr_from_pdf


class ComponentSSD(Component):
    _ssd_df = pd.read_csv(os.path.join(data_dir, 'crowdsourcing/ssd_manufacture.csv'))

    NAME = "SSD"

    __DISK_TYPE = 'ssd'

    def __init__(self, archetype=get_component_archetype(config["default_ssd"], "ssd"), **kwargs):
        super().__init__(archetype=archetype, **kwargs)
        self.manufacturer = Boattribute(
            default=get_arch_value(archetype, 'manufacturer', 'default'),
            min=get_arch_value(archetype, 'manufacturer', 'min'),
            max=get_arch_value(archetype, 'manufacturer', 'max')
        )
        self.capacity = Boattribute(
            unit="GB",
            default=get_arch_value(archetype, 'capacity', 'default'),
            min=get_arch_value(archetype, 'capacity', 'min'),
            max=get_arch_value(archetype, 'capacity', 'max')
        )
        self.density = Boattribute(
            unit="GB/cm2",
            complete_function=self._complete_density,
            default=get_arch_value(archetype, 'density', 'default'),
            min=get_arch_value(archetype, 'density', 'min'),
            max=get_arch_value(archetype, 'density', 'max')
        )
        self.layers = Boattribute(
            default=get_arch_value(archetype, 'layers', 'default'),
            min=get_arch_value(archetype, 'layers', 'min'),
            max=get_arch_value(archetype, 'layers', 'max')
        )

    def _complete_density(self):
        sub = self._ssd_df
        if self.manufacturer.has_value():
            corrected_manufacturer = fuzzymatch_attr_from_pdf(self.manufacturer.value, "manufacturer", sub)
            if corrected_manufacturer is not None:
                sub = sub[sub['manufacturer'] == corrected_manufacturer]
                if corrected_manufacturer != self.manufacturer.value:
                    self.manufacturer.set_changed(corrected_manufacturer)

        if self.layers.has_value():
            sub = sub[sub['layers'] == self.layers.value]

        if len(sub) == 1:
            self.density.set_completed(float(sub['density']), source=str(sub['source'].iloc[0]),
                                       min=float(sub['density']), max=float(sub['density']))

        elif (len(sub) == 0 or len(sub) == len(self._ssd_df)) and self.density.has_value():
            return

        else:
            self.density.set_completed(
                float(sub['density'].mean()),
                source="Average of " + str(len(sub)) + " rows",
                min=float(sub['density'].min()),
                max=float(sub['density'].max())
            )



import os
from typing import Optional, List

import pandas as pd
from fastapi import APIRouter, Body, Query, HTTPException

from boaviztapi import config, data_dir
from boaviztapi.dto.device.iot import IoT, mapper_iot_device
from boaviztapi.service.archetype import get_iot_device_archetype
from boaviztapi.service.impacts_computation import compute_impacts
from boaviztapi.service.verbose import verbose_device

iot = APIRouter(
    prefix='/v1/iot',
    tags=['iot']
)


@iot.get('/iot_device/archetypes',
         description="")
async def iot_device_get_all_archetype_name():
    df = pd.read_csv(os.path.join(data_dir, "archetypes/iot_device.csv"))
    return df['id'].tolist()


@iot.get('/iot_device/archetype_config',
         description="")
async def get_archetype_config(archetype: str = Query(example=config["default_iot_device"])):
    archetype_config = get_iot_device_archetype(archetype)
    if not archetype_config:
        raise HTTPException(status_code=404, detail=f"{archetype} not found")
    return archetype_config


@iot.post('/iot_device', description="")
async def iot_device_impact(iot: IoT = Body(None, example=""),
                            verbose: bool = True,
                            duration: Optional[float] = config["default_duration"],
                            archetype: str = config["default_iot_device"],
                            criteria: List[str] = Query(config["default_criteria"])):
    return await device_iot_impact(iot_dto=iot,
                                   verbose=verbose,
                                   duration=duration,
                                   criteria=criteria,
                                   archetype=archetype)


@iot.get('/iot_device', description="")
async def iot_device_impact(archetype: str = config["default_iot_device"],
                            verbose: bool = True,
                            duration: Optional[float] = config["default_duration"],
                            criteria: List[str] = Query(config["default_criteria"])):
    return await device_iot_impact(iot_dto=IoT(),
                                   verbose=verbose,
                                   duration=duration,
                                   criteria=criteria,
                                   archetype=archetype)


async def device_iot_impact(iot_dto: IoT,
                            archetype: str,
                            verbose: bool,
                            duration: Optional[float] = config["default_duration"],
                            criteria: List[str] = Query(config["default_criteria"])) -> dict:
    archetype_config = get_iot_device_archetype(archetype)

    if not archetype_config:
        raise HTTPException(status_code=404, detail=f"{archetype} not found")

    device = mapper_iot_device(iot_dto, archetype=archetype_config)

    if duration is None:
        duration = device.usage.hours_life_time.value

    impacts = compute_impacts(model=device, selected_criteria=criteria, duration=duration)

    if verbose:
        return {
            "impacts": impacts,
            "verbose": verbose_device(device, selected_criteria=criteria, duration=duration)
        }

    return {"impacts": impacts}



from boaviztapi import config
from boaviztapi.model.boattribute import Boattribute
from boaviztapi.model.component.component import Component
from boaviztapi.service.archetype import get_component_archetype, get_arch_value


class ComponentHDD(Component):
    NAME = "HDD"

    __DISK_TYPE = 'hdd'

    def __init__(self, archetype=get_component_archetype(config["default_ssd"], "ssd"), **kwargs):
        super().__init__(archetype=archetype, **kwargs)

        self.capacity = Boattribute(
            unit="GB",
            default=get_arch_value(archetype, 'capacity', 'default'),
            min=get_arch_value(archetype, 'capacity', 'default'),
            max=get_arch_value(archetype, 'capacity', 'default')
        )


from typing import Optional, List, Union

from boaviztapi import config
from boaviztapi.dto import BaseDTO
from boaviztapi.model.boattribute import Status
from boaviztapi.model.usage import ModelUsage, ModelUsageServer, ModelUsageCloud
from boaviztapi.service.archetype import get_cloud_instance_archetype, get_server_archetype
from boaviztapi.service.factor_provider import get_available_countries


class WorkloadTime(BaseDTO):
    time_percentage: float = None
    load_percentage: float = None


class ElecFactors(BaseDTO):
    gwp: Optional[float] = None
    adp: Optional[float] = None
    pe: Optional[float] = None
    gwppb: Optional[float] = None
    gwppf: Optional[float] = None
    gwpplu: Optional[float] = None
    ir: Optional[float] = None
    lu: Optional[float] = None
    odp: Optional[float] = None
    pm: Optional[float] = None
    pocp: Optional[float] = None
    wu: Optional[float] = None
    mips: Optional[float] = None
    adpe: Optional[float] = None
    adpf: Optional[float] = None
    ap: Optional[float] = None
    ctue: Optional[float] = None
    ctuh_c: Optional[float] = None
    ctuh_nc: Optional[float] = None
    epf: Optional[float] = None
    epm: Optional[float] = None
    ept: Optional[float] = None


class Usage(BaseDTO):
    use_time_ratio: Optional[float] = None

    hours_life_time: Optional[float] = None

    avg_power: Optional[float] = None
    time_workload: Optional[Union[float, List[WorkloadTime]]] = None

    usage_location: Optional[str] = None
    elec_factors: Optional[ElecFactors] = ElecFactors()


class UsageServer(Usage):
    other_consumption_ratio: Optional[float] = None


class UsageCloud(UsageServer):
    instance_per_server: Optional[int] = None


def mapper_usage(usage_dto: Usage, archetype=None) -> ModelUsage:
    usage_model = ModelUsage(archetype=archetype)

    for elec_factor in usage_dto.elec_factors.__dict__.keys():
        if usage_dto.elec_factors.__dict__[elec_factor] is not None:
            usage_model.elec_factors.get(elec_factor).set_input(usage_dto.elec_factors.__dict__[elec_factor])

    if usage_dto.time_workload is not None:
        usage_model.time_workload.value = usage_dto.time_workload
        usage_model.time_workload.min = usage_dto.time_workload
        usage_model.time_workload.max = usage_dto.time_workload

        if type(usage_dto.time_workload) == float:
            usage_model.time_workload.unit = "%"
        else:
            usage_model.time_workload.unit = "(time_percentage:%, load_percentage: %)"
        usage_model.time_workload.status = Status.INPUT

    if usage_dto.avg_power is not None:
        usage_model.avg_power.set_input(usage_dto.avg_power)

    if usage_dto.hours_life_time is not None:
        usage_model.hours_life_time.set_input(usage_dto.hours_life_time)

    if usage_dto.use_time_ratio is not None:
        usage_model.use_time_ratio.set_input(usage_dto.use_time_ratio)

    if usage_dto.usage_location is not None:
        if usage_dto.usage_location in get_available_countries(reverse=True):
            usage_model.usage_location.set_input(usage_dto.usage_location)
        else:
            usage_model.usage_location.set_changed(usage_model.usage_location.default)
            usage_model.usage_location.add_warning("Location not found. Default value used.")

    return usage_model


def mapper_usage_server(usage_dto: UsageServer,
                        archetype=get_server_archetype(config["default_server"]).get("USAGE")) -> ModelUsageServer:
    usage_model_server = ModelUsageServer(archetype=archetype)

    for elec_factor in usage_dto.elec_factors.__dict__.keys():
        if usage_dto.elec_factors.__dict__[elec_factor] is not None:
            usage_model_server.elec_factors.get(elec_factor).set_input(usage_dto.elec_factors.__dict__[elec_factor])

    if usage_dto.avg_power is not None:
        usage_model_server.avg_power.set_input(usage_dto.avg_power)

    if usage_dto.hours_life_time is not None:
        usage_model_server.hours_life_time.set_input(usage_dto.hours_life_time)

    if usage_dto.use_time_ratio is not None:
        usage_model_server.use_time_ratio.set_input(usage_dto.use_time_ratio)

    if usage_dto.time_workload is not None:
        usage_model_server.time_workload.set_input(usage_dto.time_workload)

    if usage_dto.usage_location is not None:
        if usage_dto.usage_location in get_available_countries(reverse=True):
            usage_model_server.usage_location.set_input(usage_dto.usage_location)
        else:
            usage_model_server.usage_location.set_changed(usage_model_server.usage_location.default)
            usage_model_server.usage_location.add_warning("Location not found. Default value used.")
    if usage_dto.other_consumption_ratio is not None:
        usage_model_server.other_consumption_ratio.set_input(usage_dto.other_consumption_ratio)

    return usage_model_server


def mapper_usage_cloud(usage_dto: UsageCloud, archetype=get_cloud_instance_archetype(config["default_cloud_instance"],
                                                                                     config[
                                                                                         "default_cloud_provider"]).get(
    "USAGE")) -> ModelUsageCloud:
    usage_model_cloud = ModelUsageCloud(archetype=archetype)

    for elec_factor in usage_dto.elec_factors.__dict__.keys():
        if usage_dto.elec_factors.__dict__[elec_factor] is not None:
            usage_model_cloud.elec_factors.get(elec_factor).set_input(usage_dto.elec_factors.__dict__[elec_factor])

    if usage_dto.avg_power is not None:
        usage_model_cloud.avg_power.set_input(usage_dto.avg_power)

    if usage_dto.hours_life_time is not None:
        usage_model_cloud.hours_life_time.set_input(usage_dto.hours_life_time)

    if usage_dto.use_time_ratio is not None:
        usage_model_cloud.use_time_ratio.set_input(usage_dto.use_time_ratio)

    if usage_dto.time_workload is not None:
        usage_model_cloud.time_workload.set_input(usage_dto.time_workload)

    if usage_dto.usage_location is not None:
        if usage_dto.usage_location in get_available_countries(reverse=True):
            usage_model_cloud.usage_location.set_input(usage_dto.usage_location)
        else:
            usage_model_cloud.usage_location.set_changed(usage_model_cloud.usage_location.default)
            usage_model_cloud.usage_location.add_warning("Location not found. Default value used.")

    if usage_dto.other_consumption_ratio is not None:
        usage_model_cloud.other_consumption_ratio.set_input(usage_dto.other_consumption_ratio)

    if usage_dto.instance_per_server is not None:
        usage_model_cloud.instance_per_server.set_input(usage_dto.instance_per_server)

    return usage_model_cloud



from fastapi import APIRouter, Body

from boaviztapi.dto.consumption_profile import ConsumptionProfileCPU
from boaviztapi.dto.consumption_profile.consumption_profile import mapper_cp_cpu
from boaviztapi.routers.openapi_doc.examples import cpu_consumption_profiles

consumption_profile = APIRouter(
    prefix='/v1/consumption_profile',
    tags=['consumption_profile']
)


@consumption_profile.post('/cpu',
                          description="cpu consumption profile generator")
async def cpu_consumption_profile(cp_dto: ConsumptionProfileCPU = Body(None, example=cpu_consumption_profiles),
                                  verbose: bool = True):
    cp, cpu = mapper_cp_cpu(cp_dto)
    result = cp.compute_consumption_profile_model(cpu_manufacturer=cpu.manufacturer.value,
                                                  cpu_model_range=cpu.model_range.value,
                                                  cpu_tdp=cpu.tdp.value)
    return result



from typing import Optional, List

from fastapi import HTTPException

from boaviztapi import config
from boaviztapi.dto.component import CPU, RAM, Disk, PowerSupply
from boaviztapi.dto.component.cpu import mapper_cpu
from boaviztapi.dto.component.disk import mapper_ssd, mapper_hdd
from boaviztapi.dto.component.other import mapper_power_supply
from boaviztapi.dto.component.ram import mapper_ram
from boaviztapi.dto.usage import UsageServer, UsageCloud
from boaviztapi.dto import BaseDTO
from boaviztapi.dto.usage.usage import mapper_usage_server, mapper_usage_cloud
from boaviztapi.model.boattribute import Status, Boattribute
from boaviztapi.model.component import ComponentCase
from boaviztapi.model.device.server import DeviceServer
from boaviztapi.model.services.cloud_instance import ServiceCloudInstance
from boaviztapi.model.usage import ModelUsage
from boaviztapi.service.archetype import get_server_archetype, get_arch_component, get_cloud_instance_archetype, \
    get_arch_value


class DeviceDTO(BaseDTO):
    pass


class ModelServer(BaseDTO):
    name: Optional[str] = None
    archetype: Optional[str] = None
    type: Optional[str] = None


class ConfigurationServer(BaseDTO):
    cpu: Optional[CPU] = None
    ram: Optional[List[RAM]] = None
    disk: Optional[List[Disk]] = None
    power_supply: Optional[PowerSupply] = None


class Server(DeviceDTO):
    model: Optional[ModelServer] = None
    configuration: Optional[ConfigurationServer] = None
    usage: Optional[UsageServer] = None


def mapper_server(server_dto: Server, archetype=get_server_archetype(config["default_server"])) -> DeviceServer:
    server_model = DeviceServer(archetype=archetype)

    server_model = device_mapper(server_dto, server_model)

    server_model.usage = mapper_usage_server(server_dto.usage or UsageServer(), archetype=get_arch_component(server_model.archetype,"USAGE"))
    complete_components_usage(server_model, server_model.usage)

    return server_model


def complete_components_usage(model: DeviceServer, usage: ModelUsage):
    complete_usage(model.cpu.usage, usage)
    complete_usage(model.case.usage, usage)
    for ram_unit in model.ram:
        complete_usage(ram_unit.usage, usage)
    for disk_unit in model.disk:
        complete_usage(disk_unit.usage, usage)


def complete_usage(usage_component, usage_device):
    if usage_device.avg_power.is_set():
        return
    for attr, val in usage_component.__iter__():
        if isinstance(val, Boattribute) and not val.is_set() and usage_device.__getattribute__(attr).is_set():
            usage_component.__setattr__(attr, usage_device.__getattribute__(attr))


class Cloud(Server):
    provider: Optional[str] = None
    instance_type: Optional[str] = None
    usage: Optional[UsageCloud] = None


def mapper_cloud_instance(cloud_dto: Cloud, archetype=get_cloud_instance_archetype(config["default_cloud_instance"], config["default_cloud_provider"])) -> ServiceCloudInstance:
    if get_server_archetype(archetype_name=get_arch_value(archetype, 'platform', 'default')) is False:
        raise HTTPException(status_code=404, detail=f"Cloud platform {get_arch_value(archetype, 'platform', 'default')} not found. Please add it to the server archetypes. For more information, please check the documentation https://doc.api.boavizta.org/contributing/server/.")

    model_cloud_instance = ServiceCloudInstance(archetype=archetype)

    model_cloud_instance.usage = mapper_usage_cloud(cloud_dto.usage or UsageCloud(), archetype=get_arch_component(model_cloud_instance.archetype, "USAGE"))

    complete_components_usage(model_cloud_instance.platform, model_cloud_instance.usage)
    complete_usage(model_cloud_instance.platform.usage, model_cloud_instance.usage)

    return model_cloud_instance


def device_mapper(device_dto, device_model):
    if device_dto.configuration is not None:
        if device_dto.configuration.cpu is not None:
            device_model.cpu = mapper_cpu(device_dto.configuration.cpu, archetype=get_arch_component(device_model.archetype, "CPU"))

        if device_dto.configuration.ram is not None:
            complete_ram = []
            for ram_dto in device_dto.configuration.ram:
                complete_ram.append(mapper_ram(ram_dto, archetype=get_arch_component(device_model.archetype, "RAM")))
            device_model.ram = complete_ram
        if device_dto.configuration.disk is not None:
            complete_disk = []
            for disk_dto in device_dto.configuration.disk:
                if disk_dto.type is None:
                    disk_dto.type = "ssd"
                if disk_dto.type.lower() == "ssd":
                    complete_disk.append(mapper_ssd(disk_dto, archetype=get_arch_component(device_model.archetype, "SSD")))
                elif disk_dto.type.lower() == "hdd":
                    complete_disk.append(mapper_hdd(disk_dto, archetype=get_arch_component(device_model.archetype, "HDD")))
            device_model.disk = complete_disk
        if device_dto.configuration.power_supply is not None:
            device_model.power_supply = mapper_power_supply(device_dto.configuration.power_supply, archetype=get_arch_component(device_model.archetype, "POWER_SUPPLY"))

    if device_dto.model is not None and device_dto.model.type is not None:
        device_model.case = ComponentCase(archetype=get_arch_component(device_model.archetype, "CASE"))
        if device_dto.model.type == "rack" or device_dto.model.type == "blade":
            device_model.case.case_type.value = device_dto.model.type
            device_model.case.case_type.status = Status.INPUT

    return device_model


import os

import pandas as pd

import boaviztapi.utils.roundit as rd
from boaviztapi import config, data_dir
from boaviztapi.model.boattribute import Boattribute
from boaviztapi.model.component.component import Component
from boaviztapi.model.consumption_profile.consumption_profile import RAMConsumptionProfileModel
from boaviztapi.model.impact import ImpactFactor
from boaviztapi.service.archetype import get_arch_value, get_component_archetype
from boaviztapi.utils.fuzzymatch import fuzzymatch_attr_from_pdf


class ComponentRAM(Component):
    NAME = "RAM"

    _ram_df = pd.read_csv(os.path.join(data_dir, 'crowdsourcing/ram_manufacture.csv'))

    def __init__(self, archetype=get_component_archetype(config["default_ram"], "ram"), **kwargs):
        super().__init__(archetype=archetype, **kwargs)

        self.process = Boattribute(
            default=get_arch_value(archetype, 'process', 'default'),
            min=get_arch_value(archetype, 'process', 'min'),
            max=get_arch_value(archetype, 'process', 'max')
        )
        self.manufacturer = Boattribute(
            default=get_arch_value(archetype, 'manufacturer', 'default'),
            min=get_arch_value(archetype, 'manufacturer', 'min'),
            max=get_arch_value(archetype, 'manufacturer', 'max')
        )
        self.capacity = Boattribute(
            unit="GB",
            default=get_arch_value(archetype, 'capacity', 'default'),
            min=get_arch_value(archetype, 'capacity', 'min'),
            max=get_arch_value(archetype, 'capacity', 'max')
        )
        self.density = Boattribute(
            complete_function=self._complete_density,
            unit="GB/cm2",
            default=get_arch_value(archetype, 'density', 'default'),
            min=get_arch_value(archetype, 'density', 'min'),
            max=get_arch_value(archetype, 'density', 'max')
        )

    # IMPACT COMPUTATION

    def model_power_consumption(self, ) -> ImpactFactor:
        self.usage.consumption_profile = RAMConsumptionProfileModel()
        self.usage.consumption_profile.compute_consumption_profile_model(ram_capacity=self.capacity.value)

        if type(self.usage.time_workload.value) in (float, int):
            self.usage.avg_power.set_completed(
                self.usage.consumption_profile.apply_consumption_profile(self.usage.time_workload.value))
        else:
            self.usage.avg_power.set_completed(
                self.usage.consumption_profile.apply_multiple_workloads(self.usage.time_workload.value))

        return ImpactFactor(
            value=rd.round_to_sigfig(self.usage.avg_power.value, 5)*self.units.value,
            min=rd.round_to_sigfig(self.usage.avg_power.value, 5)*self.units.min,
            max=rd.round_to_sigfig(self.usage.avg_power.value, 5)*self.units.max
        )

    # COMPLETION
    def _complete_density(self):
        sub = self._ram_df

        if self.manufacturer.has_value():
            corrected_manufacturer = fuzzymatch_attr_from_pdf(self.manufacturer.value, "manufacturer", sub)
            if corrected_manufacturer is not None:
                sub = sub[sub['manufacturer'] == corrected_manufacturer]
                if corrected_manufacturer != self.manufacturer.value:
                    self.manufacturer.set_changed(corrected_manufacturer)

        if self.process.has_value():
            sub = sub[sub['process'] == self.process.value]

        if len(sub) == 1:
            self.density.set_completed(
                float(sub['density']),
                source=str(sub['manufacturer'].iloc[0]),
                min=float(sub['density']),
                max=float(sub['density'])
            )

        elif (len(sub) == 0 or len(sub) == len(self._ram_df)) and self.density.has_value():
            return

        else:
            self.density.set_completed(
                float(sub['density'].mean()),
                source="Average of " + str(len(sub)) + " rows",
                min=float(sub['density'].min()),
                max=float(sub['density'].max())
            )
